'''
1. Calculate last friday of the month
    INPUT:
        year as YYYY & month as m
    USAGE:
        from calendarutil.util import last
        dt = last.last_friday_of_month(year, m)
            Ex. dt = last.last_friday_of_month(2021, 3)
    OUTPUT:
        date object

2. Calculate second friday of the month for a year
    INPUT:
        year as YYYY
    USAGE:
        from calendarutil.from_year import second
        dts = second.second_friday_of_month(year)
            Ex. dts = second.second_friday_of_month(2021)
    OUTPUT:
        list of date objects

3. Calculate first friday of the month between two date range
    INPUT:
        date1 and date2 as date object
    USAGE:
        from datetime import date
        from calendarutil.from_range import first
        dts = first.first_friday_of_month(date1, date2)
        Ex. dts = first.first_friday_of_month(date(2021, 3, 1), date(2021, 6, 1))
    OUTPUT:
        list of date objects
'''


from datetime import date
from calendarutil import util

def __validate_input(date1, date2):
    if not isinstance(date1, date):
        raise TypeError(f"{date1} should be a date object as {repr(date.today())}")
    if not isinstance(date2, date):
        raise TypeError(f"{date2} should be a date object as {repr(date.today())}")
    return (date1, date2) if date1 <= date2 else (date2, date1)

def date_of_month(date1, date2, day, position):
    date1, date2 = __validate_input(date1, date2)
    start_month = date1.month
    start_year = date1.year
    end_month = date2.month
    end_year = date2.year
    dates = list()
    for year in range(start_year, end_year+1):
        for month in range(1, 13):
            if (year == start_year and month < start_month) or \
                (year == end_year and month > end_month):
                continue
            command = "util.{p}.{p}_{d}_of_month(year, month)".format(p=position, d=day.lower())
            dt = eval(command)
            dates.append(dt)
    return dates

class first:    
    def first_monday_of_month(date1, date2):
        return date_of_month(date1, date2, 'MONDAY', 'first')

    def first_tuesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'TUESDAY', 'first')

    def first_wednesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'WEDNESDAY', 'first')

    def first_thursday_of_month(date1, date2):
        return date_of_month(date1, date2, 'THURSDAY', 'first')

    def first_friday_of_month(date1, date2):
        return date_of_month(date1, date2, 'FRIDAY', 'first')

    def first_saturday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SATURDAY', 'first')

    def first_sunday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SUNDAY', 'first')

class second:
    def second_monday_of_month(date1, date2):
        return date_of_month(date1, date2, 'MONDAY', 'second')

    def second_tuesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'TUESDAY', 'second')

    def second_wednesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'WEDNESDAY', 'second')

    def second_thursday_of_month(date1, date2):
        return date_of_month(date1, date2, 'THURSDAY', 'second')

    def second_friday_of_month(date1, date2):
        return date_of_month(date1, date2, 'FRIDAY', 'second')

    def second_saturday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SATURDAY', 'second')

    def second_sunday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SUNDAY', 'second')

class third:
    def third_monday_of_month(date1, date2):
        return date_of_month(date1, date2, 'MONDAY', 'third')

    def third_tuesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'TUESDAY', 'third')

    def third_wednesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'WEDNESDAY', 'third')

    def third_thursday_of_month(date1, date2):
        return date_of_month(date1, date2, 'THURSDAY', 'third')

    def third_friday_of_month(date1, date2):
        return date_of_month(date1, date2, 'FRIDAY', 'third')

    def third_saturday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SATURDAY', 'third')

    def third_sunday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SUNDAY', 'third')

class fourth:
    def fourth_monday_of_month(date1, date2):
        return date_of_month(date1, date2, 'MONDAY', 'fourth')

    def fourth_tuesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'TUESDAY', 'fourth')

    def fourth_wednesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'WEDNESDAY', 'fourth')

    def fourth_thursday_of_month(date1, date2):
        return date_of_month(date1, date2, 'THURSDAY', 'fourth')

    def fourth_friday_of_month(date1, date2):
        return date_of_month(date1, date2, 'FRIDAY', 'fourth')

    def fourth_saturday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SATURDAY', 'fourth')

    def fourth_sunday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SUNDAY', 'fourth')

class last:
    def last_monday_of_month(date1, date2):
        return date_of_month(date1, date2, 'MONDAY', 'last')

    def last_tuesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'TUESDAY', 'last')

    def last_wednesday_of_month(date1, date2):
        return date_of_month(date1, date2, 'WEDNESDAY', 'last')

    def last_thursday_of_month(date1, date2):
        return date_of_month(date1, date2, 'THURSDAY', 'last')

    def last_friday_of_month(date1, date2):
        return date_of_month(date1, date2, 'FRIDAY', 'last')

    def last_saturday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SATURDAY', 'last')

    def last_sunday_of_month(date1, date2):
        return date_of_month(date1, date2, 'SUNDAY', 'last')



import sys
import os

file_dir = os.path.dirname(os.path.abspath(__file__))
base_dir = os.path.join(file_dir, '..')
sys.path.append(base_dir)

from tests import *
from calendarutil.util import *

## Test case for class:first
def test_first_monday_of_month():
    assert first.first_monday_of_month(2021, 1) == fmom, "Test Failed"

def test_first_tuesday_of_month():
    assert first.first_tuesday_of_month(2021, 1) == ftum, "Test Failed"

def test_first_wednesday_of_monthh():
    assert first.first_wednesday_of_month(2021, 1) == fwem, "Test Failed"

def test_first_thursday_of_month():
    assert first.first_thursday_of_month(2021, 1) == fthm, "Test Failed"

def test_first_friday_of_month():
    assert first.first_friday_of_month(2021, 1) == ffrm, "Test Failed"

def test_first_saturday_of_month():
    assert first.first_saturday_of_month(2021, 1) == fsam, "Test Failed"

def test_first_sunday_of_month():
        assert first.first_sunday_of_month(2021, 1) == fsum, "Test Failed"

## Test case for class:second
def test_second_monday_of_month():
    assert second.second_monday_of_month(2021, 1) == smom, "Test Failed"

def test_second_tuesday_of_month():
    assert second.second_tuesday_of_month(2021, 1) == stum, "Test Failed"

def test_second_wednesday_of_monthh():
    assert second.second_wednesday_of_month(2021, 1) == swem, "Test Failed"

def test_second_thursday_of_month():
    assert second.second_thursday_of_month(2021, 1) == sthm, "Test Failed"

def test_second_friday_of_month():
    assert second.second_friday_of_month(2021, 1) == sfrm, "Test Failed"

def test_second_saturday_of_month():
    assert second.second_saturday_of_month(2021, 1) == ssam, "Test Failed"

def test_second_sunday_of_month():
    assert second.second_sunday_of_month(2021, 1) == ssum, "Test Failed"

## Test case for class:third
def test_third_monday_of_month():
    assert third.third_monday_of_month(2021, 1) == tmom, "Test Failed"

def test_third_tuesday_of_month():
    assert third.third_tuesday_of_month(2021, 1) == ttum, "Test Failed"

def test_third_wednesday_of_monthh():
    assert third.third_wednesday_of_month(2021, 1) == twem, "Test Failed"

def test_third_thursday_of_month():
    assert third.third_thursday_of_month(2021, 1) == tthm, "Test Failed"

def test_third_friday_of_month():
    assert third.third_friday_of_month(2021, 1) == tfrm, "Test Failed"

def test_third_saturday_of_month():
    assert third.third_saturday_of_month(2021, 1) == tsam, "Test Failed"

def test_third_sunday_of_month():
    assert third.third_sunday_of_month(2021, 1) == tsum, "Test Failed"

## Test case for class:fourth
def test_fourth_monday_of_month():
    assert fourth.fourth_monday_of_month(2021, 1) == frmom, "Test Failed"

def test_fourth_tuesday_of_month():
    assert fourth.fourth_tuesday_of_month(2021, 1) == frtum, "Test Failed"

def test_fourth_wednesday_of_monthh():
    assert fourth.fourth_wednesday_of_month(2021, 1) == frwem, "Test Failed"

def test_fourth_thursday_of_month():
    assert fourth.fourth_thursday_of_month(2021, 1) == frthm, "Test Failed"

def test_fourth_friday_of_month():
    assert fourth.fourth_friday_of_month(2021, 1) == frfrm, "Test Failed"

def test_fourth_saturday_of_month():
    assert fourth.fourth_saturday_of_month(2021, 1) == frsam, "Test Failed"

def test_fourth_sunday_of_month():
    assert fourth.fourth_sunday_of_month(2021, 1) == frsum, "Test Failed"

## Test case for class:last
def test_last_monday_of_month():
    assert last.last_monday_of_month(2021, 1) == lmom, "Test Failed"

def test_last_tuesday_of_month():
    assert last.last_tuesday_of_month(2021, 1) == ltum, "Test Failed"

def test_last_wednesday_of_monthh():
    assert last.last_wednesday_of_month(2021, 1) == lwem, "Test Failed"

def test_last_thursday_of_month():
    assert last.last_thursday_of_month(2021, 1) == lthm, "Test Failed"

def test_last_friday_of_month():
    assert last.last_friday_of_month(2021, 1) == lfrm, "Test Failed"

def test_last_saturday_of_month():
    assert last.last_saturday_of_month(2021, 1) == lsam, "Test Failed"

def test_last_sunday_of_month():
    assert last.last_sunday_of_month(2021, 1) == lsum, "Test Failed"



from setuptools import setup, find_packages

setup(name="calendarutil",
      version="2021.3.28",
      url="https://github.com/github-suraj/calendar-util",
      author="Suraj Jaiswal",
      author_email="surajkumarjaiswal053@gmail.com",
      description="Simple and lightweight calendar support for python datetime.date",
      packages=find_packages(),
      long_description=open('README.md').read(),
      long_description_content_type="text/markdown",
      project_urls={"Detail Usage": "https://github.com/github-suraj/calendar-util/blob/main/calendarutil/__init__.py"}
)



from datetime import MAXYEAR, MINYEAR
from calendarutil import util

def __validate_input(year):
    if not (MINYEAR <= year <= MAXYEAR):
        raise ValueError(f"year {year} is out of range, year should be between {MINYEAR} - {MAXYEAR}")

def date_of_month(year, day, position):
    __validate_input(year)
    dates = list()
    for month in range(1, 13):
        command = "util.{p}.{p}_{d}_of_month(year, month)".format(p=position, d=day.lower())
        dt = eval(command)
        dates.append(dt)
    return dates

class first:    
    def first_monday_of_month(year):
        return date_of_month(year, 'MONDAY', 'first')

    def first_tuesday_of_month(year):
        return date_of_month(year, 'TUESDAY', 'first')

    def first_wednesday_of_month(year):
        return date_of_month(year, 'WEDNESDAY', 'first')

    def first_thursday_of_month(year):
        return date_of_month(year, 'THURSDAY', 'first')

    def first_friday_of_month(year):
        return date_of_month(year, 'FRIDAY', 'first')

    def first_saturday_of_month(year):
        return date_of_month(year, 'SATURDAY', 'first')

    def first_sunday_of_month(year):
        return date_of_month(year, 'SUNDAY', 'first')

class second:
    def second_monday_of_month(year):
        return date_of_month(year, 'MONDAY', 'second')

    def second_tuesday_of_month(year):
        return date_of_month(year, 'TUESDAY', 'second')

    def second_wednesday_of_month(year):
        return date_of_month(year, 'WEDNESDAY', 'second')

    def second_thursday_of_month(year):
        return date_of_month(year, 'THURSDAY', 'second')

    def second_friday_of_month(year):
        return date_of_month(year, 'FRIDAY', 'second')

    def second_saturday_of_month(year):
        return date_of_month(year, 'SATURDAY', 'second')

    def second_sunday_of_month(year):
        return date_of_month(year, 'SUNDAY', 'second')

class third:
    def third_monday_of_month(year):
        return date_of_month(year, 'MONDAY', 'third')

    def third_tuesday_of_month(year):
        return date_of_month(year, 'TUESDAY', 'third')

    def third_wednesday_of_month(year):
        return date_of_month(year, 'WEDNESDAY', 'third')

    def third_thursday_of_month(year):
        return date_of_month(year, 'THURSDAY', 'third')

    def third_friday_of_month(year):
        return date_of_month(year, 'FRIDAY', 'third')

    def third_saturday_of_month(year):
        return date_of_month(year, 'SATURDAY', 'third')

    def third_sunday_of_month(year):
        return date_of_month(year, 'SUNDAY', 'third')

class fourth:
    def fourth_monday_of_month(year):
        return date_of_month(year, 'MONDAY', 'fourth')

    def fourth_tuesday_of_month(year):
        return date_of_month(year, 'TUESDAY', 'fourth')

    def fourth_wednesday_of_month(year):
        return date_of_month(year, 'WEDNESDAY', 'fourth')

    def fourth_thursday_of_month(year):
        return date_of_month(year, 'THURSDAY', 'fourth')

    def fourth_friday_of_month(year):
        return date_of_month(year, 'FRIDAY', 'fourth')

    def fourth_saturday_of_month(year):
        return date_of_month(year, 'SATURDAY', 'fourth')

    def fourth_sunday_of_month(year):
        return date_of_month(year, 'SUNDAY', 'fourth')

class last:
    def last_monday_of_month(year):
        return date_of_month(year, 'MONDAY', 'last')

    def last_tuesday_of_month(year):
        return date_of_month(year, 'TUESDAY', 'last')

    def last_wednesday_of_month(year):
        return date_of_month(year, 'WEDNESDAY', 'last')

    def last_thursday_of_month(year):
        return date_of_month(year, 'THURSDAY', 'last')

    def last_friday_of_month(year):
        return date_of_month(year, 'FRIDAY', 'last')

    def last_saturday_of_month(year):
        return date_of_month(year, 'SATURDAY', 'last')

    def last_sunday_of_month(year):
        return date_of_month(year, 'SUNDAY', 'last')



import calendar
from datetime import MAXYEAR, MINYEAR, date, timedelta

positions = ("first", "second", "third", "fourth", "last")
weekdays = ('MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY')

def __validate_input(year, month, day, position):
    if not (MINYEAR <= year <= MAXYEAR):
        raise ValueError(f"year {year} is out of range, year should be between {MINYEAR} - {MAXYEAR}")
    if not (1 <= month <= 12):
        raise ValueError(f"month {month} is out of range, month should be between 1 - 12")
    if position not in positions:
        raise ValueError(f"{position} not in {positions}")
    if day not in weekdays:
        raise ValueError(f"{day} not in {weekdays}")

def date_of_month(year, month, day, position):
    __validate_input(year, month, day, position)
    start, end = calendar.monthrange(year, month)
    if position == 'last':
        _date = date(year, month, end)
        delta = (_date.weekday() - eval(f"calendar.{day}")) % 7
        _date_day = _date - timedelta(days=delta)
    else:
        _date = date(year, month, 1)
        delta = (eval(f"calendar.{day}") - start) % 7 + (positions.index(position) * 7)
        _date_day = _date + timedelta(days=delta)
    return _date_day

class first:    
    def first_monday_of_month(year, month):
        return date_of_month(year, month, 'MONDAY', 'first')

    def first_tuesday_of_month(year, month):
        return date_of_month(year, month, 'TUESDAY', 'first')

    def first_wednesday_of_month(year, month):
        return date_of_month(year, month, 'WEDNESDAY', 'first')

    def first_thursday_of_month(year, month):
        return date_of_month(year, month, 'THURSDAY', 'first')

    def first_friday_of_month(year, month):
        return date_of_month(year, month, 'FRIDAY', 'first')

    def first_saturday_of_month(year, month):
        return date_of_month(year, month, 'SATURDAY', 'first')

    def first_sunday_of_month(year, month):
        return date_of_month(year, month, 'SUNDAY', 'first')

class second:
    def second_monday_of_month(year, month):
        return date_of_month(year, month, 'MONDAY', 'second')

    def second_tuesday_of_month(year, month):
        return date_of_month(year, month, 'TUESDAY', 'second')

    def second_wednesday_of_month(year, month):
        return date_of_month(year, month, 'WEDNESDAY', 'second')

    def second_thursday_of_month(year, month):
        return date_of_month(year, month, 'THURSDAY', 'second')

    def second_friday_of_month(year, month):
        return date_of_month(year, month, 'FRIDAY', 'second')

    def second_saturday_of_month(year, month):
        return date_of_month(year, month, 'SATURDAY', 'second')

    def second_sunday_of_month(year, month):
        return date_of_month(year, month, 'SUNDAY', 'second')

class third:
    def third_monday_of_month(year, month):
        return date_of_month(year, month, 'MONDAY', 'third')

    def third_tuesday_of_month(year, month):
        return date_of_month(year, month, 'TUESDAY', 'third')

    def third_wednesday_of_month(year, month):
        return date_of_month(year, month, 'WEDNESDAY', 'third')

    def third_thursday_of_month(year, month):
        return date_of_month(year, month, 'THURSDAY', 'third')

    def third_friday_of_month(year, month):
        return date_of_month(year, month, 'FRIDAY', 'third')

    def third_saturday_of_month(year, month):
        return date_of_month(year, month, 'SATURDAY', 'third')

    def third_sunday_of_month(year, month):
        return date_of_month(year, month, 'SUNDAY', 'third')

class fourth:
    def fourth_monday_of_month(year, month):
        return date_of_month(year, month, 'MONDAY', 'fourth')

    def fourth_tuesday_of_month(year, month):
        return date_of_month(year, month, 'TUESDAY', 'fourth')

    def fourth_wednesday_of_month(year, month):
        return date_of_month(year, month, 'WEDNESDAY', 'fourth')

    def fourth_thursday_of_month(year, month):
        return date_of_month(year, month, 'THURSDAY', 'fourth')

    def fourth_friday_of_month(year, month):
        return date_of_month(year, month, 'FRIDAY', 'fourth')

    def fourth_saturday_of_month(year, month):
        return date_of_month(year, month, 'SATURDAY', 'fourth')

    def fourth_sunday_of_month(year, month):
        return date_of_month(year, month, 'SUNDAY', 'fourth')

class last:
    def last_monday_of_month(year, month):
        return date_of_month(year, month, 'MONDAY', 'last')

    def last_tuesday_of_month(year, month):
        return date_of_month(year, month, 'TUESDAY', 'last')

    def last_wednesday_of_month(year, month):
        return date_of_month(year, month, 'WEDNESDAY', 'last')

    def last_thursday_of_month(year, month):
        return date_of_month(year, month, 'THURSDAY', 'last')

    def last_friday_of_month(year, month):
        return date_of_month(year, month, 'FRIDAY', 'last')

    def last_saturday_of_month(year, month):
        return date_of_month(year, month, 'SATURDAY', 'last')

    def last_sunday_of_month(year, month):
        return date_of_month(year, month, 'SUNDAY', 'last')



import sys
import os
import unittest

file_dir = os.path.dirname(os.path.abspath(__file__))
base_dir = os.path.join(file_dir, '..')
sys.path.append(base_dir)

from tests import *
from calendarutil.util import *

class UtilFirst(unittest.TestCase):
    def test_first_monday_of_month(self):
        self.assertEqual(first.first_monday_of_month(2021, 1), fmom)

    def test_first_tuesday_of_month(self):
        self.assertEqual(first.first_tuesday_of_month(2021, 1), ftum)

    def test_first_wednesday_of_monthh(self):
        self.assertEqual(first.first_wednesday_of_month(2021, 1), fwem)

    def test_first_thursday_of_month(self):
        self.assertEqual(first.first_thursday_of_month(2021, 1), fthm)

    def test_first_friday_of_month(self):
        self.assertEqual(first.first_friday_of_month(2021, 1), ffrm)

    def test_first_saturday_of_month(self):
        self.assertEqual(first.first_saturday_of_month(2021, 1), fsam)

    def test_first_sunday_of_month(self):
        self.assertEqual(first.first_sunday_of_month(2021, 1), fsum)

class UtilSecond(unittest.TestCase):
    def test_second_monday_of_month(self):
        self.assertEqual(second.second_monday_of_month(2021, 1), smom)

    def test_second_tuesday_of_month(self):
        self.assertEqual(second.second_tuesday_of_month(2021, 1), stum)

    def test_second_wednesday_of_monthh(self):
        self.assertEqual(second.second_wednesday_of_month(2021, 1), swem)

    def test_second_thursday_of_month(self):
        self.assertEqual(second.second_thursday_of_month(2021, 1), sthm)

    def test_second_friday_of_month(self):
        self.assertEqual(second.second_friday_of_month(2021, 1), sfrm)

    def test_second_saturday_of_month(self):
        self.assertEqual(second.second_saturday_of_month(2021, 1), ssam)

    def test_second_sunday_of_month(self):
        self.assertEqual(second.second_sunday_of_month(2021, 1), ssum)

class UtilThird(unittest.TestCase):
    def test_third_monday_of_month(self):
        self.assertEqual(third.third_monday_of_month(2021, 1), tmom)

    def test_third_tuesday_of_month(self):
        self.assertEqual(third.third_tuesday_of_month(2021, 1), ttum)

    def test_third_wednesday_of_monthh(self):
        self.assertEqual(third.third_wednesday_of_month(2021, 1), twem)

    def test_third_thursday_of_month(self):
        self.assertEqual(third.third_thursday_of_month(2021, 1), tthm)

    def test_third_friday_of_month(self):
        self.assertEqual(third.third_friday_of_month(2021, 1), tfrm)

    def test_third_saturday_of_month(self):
        self.assertEqual(third.third_saturday_of_month(2021, 1), tsam)

    def test_third_sunday_of_month(self):
        self.assertEqual(third.third_sunday_of_month(2021, 1), tsum)

class UtilFourth(unittest.TestCase):
    def test_fourth_monday_of_month(self):
        self.assertEqual(fourth.fourth_monday_of_month(2021, 1), frmom)

    def test_fourth_tuesday_of_month(self):
        self.assertEqual(fourth.fourth_tuesday_of_month(2021, 1), frtum)

    def test_fourth_wednesday_of_monthh(self):
        self.assertEqual(fourth.fourth_wednesday_of_month(2021, 1), frwem)

    def test_fourth_thursday_of_month(self):
        self.assertEqual(fourth.fourth_thursday_of_month(2021, 1), frthm)

    def test_fourth_friday_of_month(self):
        self.assertEqual(fourth.fourth_friday_of_month(2021, 1), frfrm)

    def test_fourth_saturday_of_month(self):
        self.assertEqual(fourth.fourth_saturday_of_month(2021, 1), frsam)

    def test_fourth_sunday_of_month(self):
        self.assertEqual(fourth.fourth_sunday_of_month(2021, 1), frsum)

class UtilLast(unittest.TestCase):
    def test_last_monday_of_month(self):
        self.assertEqual(last.last_monday_of_month(2021, 1), lmom)

    def test_last_tuesday_of_month(self):
        self.assertEqual(last.last_tuesday_of_month(2021, 1), ltum)

    def test_last_wednesday_of_monthh(self):
        self.assertEqual(last.last_wednesday_of_month(2021, 1), lwem)

    def test_last_thursday_of_month(self):
        self.assertEqual(last.last_thursday_of_month(2021, 1), lthm)

    def test_last_friday_of_month(self):
        self.assertEqual(last.last_friday_of_month(2021, 1), lfrm)

    def test_last_saturday_of_month(self):
        self.assertEqual(last.last_saturday_of_month(2021, 1), lsam)

    def test_last_sunday_of_month(self):
        self.assertEqual(last.last_sunday_of_month(2021, 1), lsum)
